import os
import pygame

class Actionbar(object):
    def __init__(self, grid):
        self.grid = grid
        self.height = 30
        self.posy = 0

    def set_width(self, width):
        self.width= width

    def set_posy(self, posy):
        self.posy = posy

    def draw(self):
        """
                Draws the action bar
                """
        package_directory = os.path.dirname(os.path.abspath(__file__))
        myfont = pygame.font.SysFont("monospace", 15)
        actionbar = pygame.Surface((self.width, 30))
        actionbar.fill((255, 255, 255))
        # Act Button:
        path = os.path.join(package_directory, "data", 'play.png')
        image = pygame.image.load(path)
        image = pygame.transform.scale(image, (20, 20))
        actionbar.blit(image, (5, 5))
        label = myfont.render("Act", 1, (0, 0, 0))
        actionbar.blit(label, (30, 5))
        # Run Button:
        if not self.grid.is_running:
            path = os.path.join(package_directory, "data", 'run.png')
            image = pygame.image.load(path)
            image = pygame.transform.scale(image, (20, 20))
            actionbar.blit(image, (60, 5))
            label = myfont.render("Run", 1, (0, 0, 0))
            actionbar.blit(label, (85, 5))
        if self.grid.is_running:
            path = os.path.join(package_directory, "data", 'pause.png')
            image = pygame.image.load(path)
            image = pygame.transform.scale(image, (20, 20))
            actionbar.blit(image, (60, 5))
            label = myfont.render("Pause", 1, (0, 0, 0))
            actionbar.blit(label, (85, 5))
        # Reset Button:
        path = os.path.join(package_directory, "data", 'reset.png')
        image = pygame.image.load(path)
        image = pygame.transform.scale(image, (20, 20))
        actionbar.blit(image, (140, 5))
        label = myfont.render("Reset", 1, (0, 0, 0))
        actionbar.blit(label, (165, 5))
        # Info-Button
        path = os.path.join(package_directory, "data", 'question.png')
        image = pygame.image.load(path)
        image = pygame.transform.scale(image, (20, 20))
        actionbar.blit(image, (225, 5))
        label = myfont.render("Info", 1, (0, 0, 0))
        actionbar.blit(label, (245, 5))
        # Info-Button
        path = os.path.join(package_directory, "data", 'left.png')
        image = pygame.image.load(path)
        image = pygame.transform.scale(image, (20, 20))
        actionbar.blit(image, (285, 5))
        label = myfont.render("Speed:" + str(self.grid.speed), 1, (0, 0, 0))
        actionbar.blit(label, (305, 5))
        path = os.path.join(package_directory, "data", 'right.png')
        image = pygame.image.load(path)
        image = pygame.transform.scale(image, (20, 20))
        actionbar.blit(image, (380, 5))
        pygame.screen.blit(actionbar,(0, self.posy, self.width, self.height))




# -*- coding: utf-8 -*-
"""
Created on Mon Apr 16 21:49:29 2018

@author: asieb
"""
import logging
import os

import pygame
from gamegridp import keys
from gamegridp import gamegrid_toolbar
from gamegridp import gamegrid_console
from gamegridp import gamegrid_actionbar


class GameGrid(object):
    """The Main GameGrid."""

    grid = None

    def __init__(self, title, cell_size=32,
                 columns=8, rows=8, margin=0,
                 background_color=(255, 255, 255), cell_color=(0, 0, 0),
                 img_path=None, img_action="upscale", speed=60, toolbar=False, console=False, actionbar=True):
        self.grid = self
        self.__is_setting_up__ = False
        # Define instance variables
        self.title = title  #
        self._logging = logging.getLogger('GameGrid')
        self.__done__ = False
        self._image = None
        self._grid = []
        self._info = False
        self._actors = []
        self._frame = 0
        self.colliding_actors = []
        self._key_holding_allowed = False
        self._resolution = ()
        self._running = False
        self._cell_margin = 0
        self._collision_type = "cell"
        self._image = None
        self._cell_transparency = 0
        self._clock = None
        self._grid_rows = 0
        self._grid_columns = 0
        self._background_color = (255, 255, 255)
        self._cell_color = (0, 0, 0)
        self._draw_queue = []  # a queue of rectangles
        self._key_pressed = False
        self._key = 0
        self._speed = speed
        self._animated = False
        self._show_bounding_boxes = False
        self._show_direction_marker = False
        if toolbar is True:
            self.toolbar = gamegrid_toolbar.Toolbar(self)
        else:
            self.toolbar = None
        if actionbar is True:
            self.actionbar = gamegrid_actionbar.Actionbar(self)
        else:
            self.actionbar = None
        if console is True:
            self.console = gamegrid_console.Console(self, 5)
        else:
            self.console = None
        """
        Initialises the grid
        """
        # attributes
        self._cell_margin = margin
        self._grid_columns = columns
        self._grid_rows = rows
        self._cell_size = cell_size
        if self.cell_size == 1:
            self._key_holding_allowed = True
            self._collision_type = "bounding_box"
        else:
            self._key_holding_allowed = False
            self._collision_type = "bounding_box"
        self._background_color = background_color
        self._cell_color = cell_color
        # grid and grid-dimensions
        for row in range(rows):
            self._grid.append([])
            for column in range(columns):
                self._grid[row].append(0)
        # Init gui
        if self.toolbar is not None:
            _toolbar_width = self.toolbar.width
        else:
            _toolbar_width = 0
        if self.actionbar is not None:
            _actionbar_height = self.actionbar.height
        else:
            _actionbar_height = 0
        if self.console is not None:
            _console_height = self.console.height
        else:
            _console_height = 0
        x_res = self.__grid_width_in_pixels__ + _toolbar_width
        y_res = self.__grid_height_in_pixels__ + _actionbar_height + _console_height  # 100 pixels for actionbar
        if self.toolbar is not None:
            self.toolbar.set_height(y_res)
            self.toolbar.set_posx(self.__grid_width_in_pixels__ )
        if self.actionbar is not None:
            self.actionbar.set_width(x_res)
            self.actionbar.set_posy(self.__grid_height_in_pixels__)
        if self.console is not None:
            self.console.set_width(x_res)
            self.console.set_posy(self.__grid_height_in_pixels__+_actionbar_height)
        self._resolution = x_res, y_res
        WINDOW_SIZE = (self._resolution[0], self._resolution[1])
        self._logging.info(
            "gamegrid.__init__(): Created windows of dimension: (" + str(self._resolution[0]) + "," + str(
                self._resolution[1]) + ")")
        # Init pygame
        pygame.screen = pygame.display.set_mode(WINDOW_SIZE)
        self.grid_surface = pygame.Surface((self.__grid_width_in_pixels__,self.__grid_height_in_pixels__))
        self.grid_surface.fill((255,255,255))
        pygame.display.set_caption(title)
        pygame.screen.fill((255, 255, 255))
        # Init clock
        self.clock = pygame.time.Clock()
        # time of last frame change
        self.last_update = pygame.time.get_ticks()
        # current frame
        self._frame = 0
        pygame.init()
        # image
        self.set_image(img_path, img_action)
        self._setup()
        # Draw_Qeue
        pygame.screen.blit(self.grid_surface, (0, 0, self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
        self._draw_queue.append(pygame.Rect(0, 0, self._resolution[0], self._resolution[1]))

    def add_cell_image(self, img_path: str, location: tuple):
        top_left = self.cell_to_pixel(location)
        cell_image = pygame.image.load(img_path).convert()
        cell_image = pygame.transform.scale(cell_image, (self.cell_size, self.cell_size))
        self._image.blit(cell_image, (top_left[0], top_left[1], self.cell_size, self.cell_size))

    def set_image(self, img_path: str, img_action: str = "upscale", size=None):
        self._logging.info(
            "gamegrid.set_image : Set new image with action:" + str(img_action) + " and path:" + str(img_path))
        if img_path is not None:
            self._image = pygame.image.load(img_path).convert()
            self._image = self._image
            self._cell_transparency = 0
            if img_path is not None and img_action == "scale":
                if size is None:
                    self._image = pygame.transform.scale(self._image,
                                                         (
                                                         self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
                else:
                    self._image = pygame.transform.scale(self._image, (size[0], size[1]))
                self._image = pygame.transform.scale(
                    self._image, (self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
            elif img_path is not None and img_action == "upscale":
                if self._image.get_width() < self.__grid_width_in_pixels__ or self._image.get_height() < self.__grid_height_in_pixels__:
                    self._image = pygame.transform.scale(
                        self._image, (self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
            elif img_path is not None and img_action == "fill":
                if size is None:
                    self._image = pygame.transform.scale(self._image,
                                                         (self.cell_size, self.cell_size))
                else:
                    self._image = pygame.transform.scale(self._image, (size[0], size[1]))
                i = 0
                j = 0
                surface = pygame.Surface((self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
                while (i < self.__grid_width_in_pixels__):
                    j = 0
                    while (j < self.__grid_height_in_pixels__):
                        surface.blit(self._image, (i, j), (0, 0, self._image.get_width(),
                                                           self._image.get_height()))
                        j = j + self._image.get_height() + self.cell_margin
                    i = i + self._image.get_width() + self.cell_margin
                self._image = surface
        else:
            self._cell_transparency = None
            self._image = pygame.Surface((self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))

        # Crop surface
        cropped_surface = pygame.Surface((self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
        cropped_surface.fill((255, 255, 255))
        cropped_surface.blit(self._image, (0, 0),
                             (0, 0, self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
        self._image = cropped_surface



        # draw grid around the cells
        if self._cell_margin > 0:
            i = 0
            while i <= self.__grid_width_in_pixels__:
                pygame.draw.rect(self._image, self._background_color,
                                 [i, 0, self._cell_margin, self.__grid_height_in_pixels__])
                i += self.cell_size + self._cell_margin
            i = 0
            while i <= self.__grid_height_in_pixels__:
                pygame.draw.rect(self._image, self._background_color,
                                 [0, i, self.__grid_width_in_pixels__, self._cell_margin])
                i += self.cell_size + self._cell_margin
        self._draw_queue.append(pygame.Rect(0, 0, self._resolution[0], self._resolution[1]))

    def act(self):
        """
        Should be overwritten in sub-classes. Act is called in every cycle of the mainloop
        """
        pass

    def add_actor(self, actor, location=None):
        """
        Adds an actor to the grid.
        The method is called when a new actor is created.
        """
        self._actors.append(actor)
        self.repaint_area(actor.image_rect)
        if location is not None:
            actor.location = location

    def __act_all__(self):
        """
        act_all() is called in every cycle of the game loop.
        It acts in the following order:
          1. All actors act
          2. The grid acts
        :param grid_act: Does the grid act
        """
        for actor in self._actors:
            actor.act()
        self.act()

    @property
    def cell_size(self):
        """"""
        return self._cell_size

    @cell_size.setter
    def cell_size(self, value: int):
        """ Sets the cell-size"""
        self._cell_size = value



    @property
    def cell_margin(self):
        """
        returns the margin between cells
        """
        return self._cell_margin

    def draw(self):
        """
        draws the entire window with grid, actionbar and toolbar
        :return:
        """
        self.__draw_grid__()
        self.__draw_actionbar__()
        self.__draw_toolbaar__()
        self.__draw_console__()

    def __draw_grid__(self):
        """
        Draws grid with all actors in it.
        """
        # Draw the gamegrid-cells
        if self._image is not None:
            self.grid_surface.blit(self._image, (0, 0, self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))
        for actor in self._actors:
            actor.next_sprite()
            actor.draw()
        pygame.screen.blit(self.grid_surface, (0, 0, self.__grid_width_in_pixels__, self.__grid_height_in_pixels__))

    def __draw_toolbaar__(self):
        if self.toolbar is not None:
            self.toolbar.draw()
            self.grid._draw_queue.append(pygame.Rect(0, 0, self._resolution[0], self._resolution[1]))

    def __draw_console__(self):
        if self.console is not None:
            self.console.draw()
            self.grid._draw_queue.append(pygame.Rect(0, 0, self._resolution[0], self._resolution[1]))

    def __draw_actionbar__(self):
        if self.actionbar is not None:
            self.actionbar.draw()
            self.grid._draw_queue.append(pygame.Rect(0, 0, self._resolution[0], self._resolution[1]))


    @property
    def actors(self) -> set:
        """
        returns all actors in grid
        :return: a list of all actors
        """
        return self._actors

    @property
    def frame(self) -> int:
        """
        Returns the actual frame
        :return: the value of actual frame
        """
        return self._frame


    @property
    def __grid_width_in_pixels__(self) -> int:
        """ returns the grid with in pixes
        :return: The grid_width in pixels
        """
        return self._grid_columns * self.cell_size + (self._grid_columns + 1) * self._cell_margin

    @property
    def __grid_height_in_pixels__(self) -> int:
        """ returns the grid-height in pixels"""
        height = self._grid_rows * self.cell_size + (self._grid_rows + 1) * self._cell_margin
        return height

    def __grid_dimensions_in_pixels__(self) -> tuple:
        """
        Returns the grid-dimension in pixesls
        :return:
        """
        return self.__grid_width_in_pixels__, self.__grid_height_in_pixels__

    def __check_collision_cell__(self, actor1, actor2) -> bool:
        """
        Checks if two actors are in the same cell
        :param actor1: first actor
        :param actor2: partner
        :return: true if both actors are on the same cell
        """
        if actor1.is_in_grid(self) and actor2.is_in_grid(self):
            self._logging.debug(
                "gamegrid.collision_cell() : Collision? A1:" + str(actor1.location) + ",A2:" + str(actor2.location))
            if actor1.location == actor2.location:
                return True
            else:
                return False

    def __check_collision_bounding_box__(self, actor1, actor2) -> bool:
        """
        Checks if the bounding boxes of two actors collide
        :param actor1: first actor
        :param actor2: partner
        :return: true if the bounding boxes collide
        """
        if actor1.bounding_box().colliderect(actor2.bounding_box()):
            self._logging.debug("gamegrid.collision_bounding_box: colliding")
            return True
        else:
            self._logging.debug("gamegrid.collision_bounding_box: not colliding")
            return False

    def __get_bounding_box_collisions__(self, actor):
        """
        Gets all bounding box collisions for one actor
        :param actor:
        :return:
        """
        for partner in actor._collision_partners:
            if self.__check_collision_bounding_box__(actor, partner):
                return partner
        return None

    def __get_cell_collisions__(self, actor):
        """
        Gets all cell collisions for one actor
        :param actor:
        :return:
        """
        for partner in actor._collision_partners:
            if self.__check_collision_cell__(actor, partner):
                return partner
        return None

    def __collision__(self):
        self._logging.debug("gamegrid.__collision__() - Type:" + str(self._collision_type))
        pairs = []
        for actor in self.actors:
            if self._collision_type == "bounding_box":
                partner2 = self.__get_bounding_box_collisions__(actor)
                if partner2 is not None:
                    partner1 = actor
                    if (partner2, partner1) not in pairs:
                        pairs.append((partner1, partner2))
            else:
                partner2 = self.__get_cell_collisions__(actor)
                if partner2 is not None:
                    partner1 = actor
                    if (partner2, partner1) not in pairs:
                        pairs.append((partner1, partner2))
        for pair in pairs:
            partner1 = pair[0]
            partner2 = pair[1]
            if pair not in self.colliding_actors:
                self.collision(partner1, partner2)
        self.colliding_actors = pairs

    def bounce(self, partner1, partner2):
        mirror_axis = (partner1.direction + partner2.direction) / 2
        self._logging.info(
            "Bouncing: actor1:" + str(partner1.direction) + ", actor2:" + str(partner2.direction) + "mirror:" + str(
                mirror_axis))
        self._logging.info(
            "Bouncing: actor2:" + str(partner2.direction) + ", actor2:" + str(partner1.direction) + "mirror:" + str(
                mirror_axis))
        partner1.bounce_against_line(mirror_axis)
        partner2.bounce_against_line(mirror_axis)

    def collision(self, partner1, partner2):
        """
        overwritten by main_method
        :return:
        """

    def get_actors_at_location(self, location: tuple, class_name: str = "") -> list:
        """
        Get all actors at a specific location
        """
        actors_at_location = []
        for actor in self._actors:
            if actor.location == location:
                if class_name is "" or actor.__class__.__name__ == class_name:
                    actors_at_location.append(actor)
        return actors_at_location

    def is_location_in_grid(self, location):
        if location[0] > self._grid_columns - 1:
            return False
        elif location[1] > self._grid_rows - 1:
            return False
        elif location[0] < 0 or location[1] < 0:
            return False
        else:
            return True

    def is_at_left_border(self, rect):
        if rect.topleft[0] <= 0:
            return True
        else:
            return False

    def is_at_bottom_border(self, rect):
        if rect.topleft[1] + rect.height >= self.__grid_height_in_pixels__:
            return True
        else:
            return False

    def is_at_right_border(self, rect):
        if rect.topleft[0] + rect.width >= self.__grid_width_in_pixels__:
            return True
        else:
            return False

    def is_at_top_border(self, rect):
        if rect.topleft[1] <= 0:
            return True
        else:
            return False

    def is_rectangle_in_grid(self, rect):
        if rect.topleft[0] < 0 \
                or rect.topleft[1] < 0 \
                or rect.topleft[0] + rect.width > self.__grid_width_in_pixels__ \
                or rect.topleft[1] + rect.height > self.__grid_height_in_pixels__:
            self._logging.info("is_rectangle_in_grid() : false)")
            return False  # rectangle is not in grid
        else:
            self._logging.info("is_rectangle_in_grid() : true)")
            return True  # rectangle is in grid

    def is_empty_cell(self, cell: tuple) -> bool:
        if not self.get_actors_at_location(cell):
            return True
        else:
            return False

    def __listen__(self):
        """
        Listen to key events
        """
        if self.__is_setting_up__:
            return
        key_pressed = False
        self._logging.debug("gamegrid.__listen__() : Listen...")
        do_act = False
        for event in pygame.event.get():
            key = False
            cell = False
            # Event: Quit
            if event.type == pygame.QUIT:
                self.__done__ = True
                pygame.quit()
                os._exit(0)
            # Event: Mouse-Button Down
            elif event.type == pygame.MOUSEBUTTONDOWN:
                pos = pygame.mouse.get_pos()
                self._logging.info("gamegrid.__listen__(): Mouseclick with button:" + str(event.button))
                # Click is in gamegrid
                if pos[0] < self.__grid_width_in_pixels__ and pos[
                    1] < self.__grid_height_in_pixels__ and event.button == 1:
                    cell_location = self.__pixel_to_cell__(pos)
                    self._logging.info("gamegrid.__listen__() : " + str(cell_location))
                    column = cell_location[0]
                    row = cell_location[1]
                    cell_clicked = (column, row)
                    self.__listen_all__("mouse_left", cell_location)
                    self._logging.info("gamegrid.__listen__(): Mouseclick left at grid-position:" +
                                       str(cell_clicked))
                elif pos[0] < self.__grid_width_in_pixels__ and pos[
                    1] < self.__grid_height_in_pixels__ and event.button == 3:
                    cell_location = self.__pixel_to_cell__(pos)
                    self._logging.info("gamegrid.__listen__() : " + str(cell_location))
                    column = cell_location[0]
                    row = cell_location[1]
                    cell_clicked = (column, row)
                    self._logging.info("gamegrid.__listen__(): - Mouseclick right at grid-position:" +
                                       str(cell_clicked))
                    self.__listen_all__("mouse_right", cell_location)
                # Click is in status_bar
                elif pos[1] >= self.__grid_height_in_pixels__:
                    if pos[0] > 5 and pos[0] < 60:
                        if not self._running:
                            self._logging.debug("gamegrid.__listen__(): : Act")
                            return True
                    elif pos[1] >= self.__grid_height_in_pixels__ and pos[0] > 60 and pos[
                        0] < 120 and not self._running:
                        self._running = True
                        self._logging.debug("gamegrid.__listen__(): : Play")
                    elif pos[1] >= self.__grid_height_in_pixels__ and pos[0] > 60 and pos[0] < 120 and self._running:
                        self._running = False
                        self._logging.debug("gamegrid.__listen__() : Pause")
                    elif pos[1] >= self.__grid_height_in_pixels__ and pos[0] > 120 and pos[0] < 220:
                        self._logging.info("gamegrid.__listen__() : Reset")
                        self._running = False
                        self.reset()
                    elif pos[1] >= self.__grid_height_in_pixels__ and pos[0] > 220 and pos[0] < 280:
                        self._logging.info("gamegrid.__listen__() : Info")
                        if self._info == True:
                            self._show_bounding_boxes = False
                            self._show_direction_marker = False
                            self._info = False
                        elif self._info == False:
                            self._show_bounding_boxes = True
                            self._show_direction_marker = True
                            self._info = True
                    elif pos[1] >= self.__grid_height_in_pixels__ and pos[0] > 285 and pos[0] < 345:
                        self.speed = self.speed - 1
                    elif pos[1] >= self.__grid_height_in_pixels__ and pos[0] > 345 and pos[0] < 395:
                        self.speed = self.speed + 1
                elif pos[0] > self.__grid_width_in_pixels__:
                    toolbar_event = self.toolbar.listen("mouse_left", position = (pos[0], pos[1]))
                    self.__listen_all__(toolbar_event)
            elif event.type == pygame.KEYDOWN:
                keys_pressed = pygame.key.get_pressed()
                self.__listen_all__("key_down", keys.key_pressed_to_key(keys_pressed))
        if pygame.key.get_pressed().count(1) != 0:
            keys_pressed = pygame.key.get_pressed()
            self.__listen_all__("key", keys.key_pressed_to_key(keys_pressed))
        mouse_x, mouse_y = pygame.mouse.get_pos()
        if mouse_x> self.__grid_width_in_pixels__:
            toolbar_event = self.toolbar.listen("mouse_hover", position=(mouse_x, mouse_y))

        return False

    def __listen_all__(self, event: str = None, data=None):
        """ Calls listen of grid and all actors """
        self.listen(event, data)
        for actor in self._actors:
            actor.listen(event, data)

    def listen(self, event: str = None, data=None):
        """
        should be overwritten in sub classes
        :param event: The event which was triggered (e.g. mouse-left, key, mouse-right, button...
        :param data: The data submitted by the event (e.g. cell-location
        """
        pass

    def __update__(self, no_logic: bool = False, no_drawing: bool = False):
        ''' Part 1:
        For grid an all actors
        listen to events
        react with listen() method
        '''
        if self.__is_setting_up__:
            no_logic = True
            no_drawing = True
            max_speed = 120
        else:
            max_speed = self._speed
        if not no_logic:
            do_act = self.__listen__()
            ''' Part 2:
                For grid an all actors
                act()
            '''
            if (self._running or do_act) and (not no_logic):
                self.__act_all__()
        ''' Part 3: Draw actors
            Draw actors in every frame, regardless of speed
        '''
        if not no_logic:
            self.__collision__()
        if not no_drawing:
            self.draw()
            pygame.display.update(self._draw_queue)
            self._draw_queue = []
        self._logging.debug("gamegrid.update() : str(self.clock.tick()")
        if not no_logic:
            self.clock.tick(max_speed)
            self._logging.debug("gamegrid.update() frame:" + str(self._frame) + "speed:" + str(max_speed) + "tick:")
        if self._frame % max_speed == 0:
            self._frame = 1
        else:
            self._frame = self._frame + 1

    def repaint_area(self, rect: pygame.Rect):
        """
        Repaints area on next update
        :param rect: The rectangle which should be redrawn
        """
        self._draw_queue.append(pygame.Rect(rect))

    def remove_actor(self, actor=None, cell: tuple = None):
        """
        Removes actor from grid
        :param actor: Removes the actor from the grid
        :param cell: Removes all actors in one cell from the grid
        :return:
        """
        if cell == None:
            actor.remove()
        else:
            actors_at_cell = self.get_actors_at_location(cell)
            self._logging.info("gamegrid.remove_actor(): Remove actor at: " + str(cell))
            for actor in actors_at_cell:
                try:
                    self._logging.info("gamegrid.remove_actor() : remove_actor" + str(actor) + " wird gelöscht...")
                    self._actors.remove(actor)
                    actor._remove_from_grid()
                except ValueError:
                    self._logging.warning("gamegrid.remove_actor() : Nicht in Liste vorhanden")

    def remove_all_actors(self):
        """ remove all actors from the grid
        """
        for actor in self._actors:
            self.remove_actor(actor)
        self._actors = []

    def reset(self):
        """ resets the grid:
        Sets all actors to their starting positions.
        May not work if custom variables where set
        """
        self.remove_all_actors()
        self._setup()
        self._draw_queue.append(pygame.Rect(0, 0, self._resolution[0], self._resolution[1]))
        self.__update__(no_logic=True, no_drawing=True)

    def stop(self):
        self._running = False

    def run(self):
        self._running = True

    @property
    def is_running(self):
        return self._running

    @is_running.setter
    def is_running(self, value):
        self._running = value

    def show(self):
        """
        Starts the mainloop
        """
        # Start Mainloop
        self.__update__()
        while not self.__done__:
            self.__update__()
        pygame.quit()

    def _setup(self):
        self.__is_setting_up__ = True
        self.setup()
        self.__is_setting_up__ = False

    def setup(self):
        """
        should be overwritten in sub_classes
        """
        pass

    @property
    def speed(self):
        return self._speed

    @speed.setter
    def speed(self, value):
        self._speed = value

    def __pixel_to_cell__(self, pos: tuple):
        """
        transforms a pixel-coordinate into a cell coordinate in grid
        :param pos: the position in pixels
        """
        column = \
            (pos[0] - self._cell_margin) // (self.cell_size + self._cell_margin)
        row = \
            (pos[1] - self._cell_margin) // (self.cell_size + self._cell_margin)
        return column, row

    def cell_rect(self, cell: tuple):
        """
        gets the rectangle which describes the given cell
        """
        x = cell[0] * self.cell_size + cell[0] * self._cell_margin + self._cell_margin
        y = cell[1] * self.cell_size + cell[1] * self._cell_margin + self._cell_margin
        return pygame.Rect(x, y, self.cell_size, self.cell_size)

    def cell_to_pixel(self, cell: tuple):
        """
        gets the top-left point of a cell
        """
        x = cell[0] * self.cell_size + cell[0] * self._cell_margin + self._cell_margin
        y = cell[1] * self.cell_size + cell[1] * self._cell_margin + self._cell_margin
        return pygame.Rect(x, y, self.cell_size, self.cell_size)

    def play_sound(self, sound_path):
        pygame.mixer.music.load(sound_path)
        pygame.mixer.music.play(0)


W = 119
S = 115
A = 97
D = 100
SPACE = 32
UP = 273
DOWN = 274
LEFT = 276
RIGHT = 275
Q = 113
E = 101
Y = 122
X = 120
C = 99


def key_pressed_to_key(key_pressed_list: list):
    keys = []
    for index, item in enumerate(key_pressed_list):
        if item:
            if index == W:
                keys.append("W")
            if index == S:
                keys.append("S")
            if index == A:
                keys.append("A")
            if index == D:
                keys.append("D")
            if index == UP:
                keys.append("UP")
            if index == DOWN:
                keys.append("DOWN")
            if index == LEFT:
                keys.append("LEFT")
            if index == RIGHT:
                keys.append("RIGHT")
            if index == Q:
                keys.append("Q")
            if index == E:
                keys.append("E")
            if index == Y:
                keys.append("Y")
            if index == X:
                keys.append("X")
            if index == C:
                keys.append("C")
    return keys



from gamegridp import keys
from gamegridp.actor import Actor
from gamegridp.gamegrid import GameGrid
from gamegridp.gamegrid_toolbar import Toolbar
from gamegridp.gamegrid_console import Console
from gamegridp.gamegrid_actionbar import Actionbar

__all__ = ['GameGrid', 'Actor', 'Console', 'Toolbar', 'Actionbar', 'keys']



# -*- coding: utf-8 -*-
"""
Created on Mon Apr 16 21:50:48 2018

@author: asieb
"""

import logging
import math

import pygame


class Actor(object):

    def __init__(self, grid, location: list = (0, 0), color: tuple = (0, 0, 255), title: str = "Actor",
                 img_path: str = None, size: tuple = (40, 40), img_action: str = None):
        # define instance variables
        self.title = title
        self._logging = logging.getLogger('Actor:')
        # Define instance variables
        self._original_images = []  # All images stores for actor
        self._image = None
        self._image_index = 0
        self._is_rotatable = False
        self._is_flipped = False
        self.__grid__ = grid
        self._location = location
        self._direction = 0
        self.color = color
        self.size = size
        self._animation_speed = 4
        self._animated = False
        self._animations = []
        self._animation = ""
        self._blocked = False
        self._collision_partners = []
        # Set Actor image
        self._logging.debug("actor.__init__() : Target-Location:" + str(self.location))
        self._has_image = False
        # Add image to actor
        if img_path is not None:
            self.add_image(img_path, img_action, size)
        else:
            self.delete_images()
        self._logging.debug(
            "actor.__init__() : Actor: " + str(title) + "'s setup wurde ausgeführt" + str(self._is_rotatable))
        # Add actor to grid
        grid.add_actor(self, location)
        # set the bounding-box style (cell for cell-based games, image for pixel-based games
        if self.grid.cell_size == 1:
            self.__bounding_box_type__ = "image"
        else:
            self.__bounding_box_type__ = "cell"
        self.__bounding_box_size__ = None
        self.setup()
        self._logging.debug("actor.__init__() : Actor " + str(title) + " wurde initialisiert")

    def act(self):
        """
        The method should be overwritten in subclasses
        """
        pass

    def set_image(self, img_path: str, img_action: str = None, size=None):
        """
        Adds an single image to an actor, deletes all other images
        :param img_path: The path of the image relative to the actual path
        :param img_action: The image action (scale, do_nothing, crop)
        :param size: scale/crop : Size as 2-Tuple
        """
        if self.has_image:
            self.delete_images()
        self.add_image(img_path, img_action, size)

    def add_image(self, img_path: str, img_action: str = None, size=None):
        """ adds an image to the actor
        :param img_path: The path of the image relative to the actual path
        :param img_action: The image action (scale, do_nothing, crop)
        :param size: scale/crop : Size as 2-Tuple
        """
        self._logging.info("add_image(): Start add image")
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        if img_action is None and self.grid.cell_size == 1:
            img_action = "do_nothing"
        if img_action is None and self.grid.cell_size > 1:
            img_action = "scale"
        self._logging.info(
            "img_action:" + str(img_action))
        if not self.has_image:
            self._original_images = []
            self._logging.info("add_image(): list was cleared:" + str(self._original_images.__len__()))
        self._logging.info("add_image(): Has image:" + str(self.has_image))
        self._original_images.append(pygame.image.load(img_path).convert_alpha())
        self._logging.info("actor.add_image() : Number of Actor images:" + str(self._original_images.__len__()))
        self.__image_transform__(-1, img_action, size)
        self.image = self._original_images[0]  # overwrite image
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        self.has_image = True

    def delete_images(self):
        self._original_images = []
        self._image = pygame.Surface(self.size)
        self._image.fill(self.color)
        self._original_images.append(self._image)
        self.has_image = False

    def animate(self, animation: str = ""):
        """
        Starts an animation
        :param animation: Type of animation (not implemented)
        """
        if not self._animated:
            self._animation = animation
            self._animated = True

    def add_collision_partner(self, partner):
        self._collision_partners.append(partner)

    def stop(self):
        """
        Stops all animations
        """
        self._animated = False

    def next_sprite(self):
        """
        Loads the next sprite in the current animation
        """
        if self._animated:
            if self.__grid__.frame % self._animation_speed == 0:
                self.image_next()

    @property
    def direction(self) -> int:
        """ Gets the direction of actor """
        return self._direction

    @direction.setter
    def direction(self, value: int):
        self._logging.debug(
            "actor.rotation() : rotated by " + str(value) + " degrees. Is rotatable?: " + str(self._is_rotatable))
        self.__rotate__(value)
        self._direction = value
        # if value < 0:
        #    self.direction = 360 + value
        # if value > 360:
        self._direction = value % 360

    def __rotate__(self, direction: int):
        """
        rotates the actor by _direction_ degrees
        :param direction:
        """
        if self.is_rotatable:
            self.__grid__.repaint_area(self.image_rect)
            # rotate the original image to new direction
            self._logging.debug("actor.__rotate: rotate Image" + str(dir(self._original_images)) + ", Image: " + str(
                self._image) + "Index:" + str(self._image_index))
            self.image = pygame.transform.rotate(self._original_images[self._image_index], direction)
            self.__grid__.repaint_area(pygame.Rect(self.image_rect))

    def is_at_border(self):
        if self.is_at_left_border():
            return "left"
        elif self.is_at_right_border():
            return "right"
        elif self.is_at_bottom_border():
            return "bottom"
        elif self.is_at_top_border():
            return "top"
        else:
            return False

    def is_at_left_border(self):
        if self.grid.is_at_left_border(self.bounding_box()):
            return True
        else:
            return False

    def is_at_bottom_border(self):
        if self.grid.is_at_bottom_border(self.bounding_box()):
            return True
        else:
            return False

    def is_at_right_border(self):
        if self.grid.is_at_right_border(self.bounding_box()):
            return True
        else:
            return False

    def is_at_top_border(self):
        if self.grid.is_at_top_border(self.bounding_box()):
            return True
        else:
            return False

    def is_near_edge(self, target: tuple = None) -> bool:
        if target == None:
            box = self.bounding_box
        else:
            box = self.bounding_box(target)
        if self.grid.is_rectangle_in_grid(box):
            return True
        else:
            return False

    def bounce_against_line(self, line_axis):
        self.direction = (line_axis * 2 - self.direction) % 360

    def bounce_from_border(self, border: str):
        if border == "top":
            deg_mirror = 0
        elif border == "bottom":
            deg_mirror = 180
        elif border == "left":
            deg_mirror = 90
        elif border == "right":
            deg_mirror = 270
        self.direction = deg_mirror * 2 - self.direction

    def is_in_grid(self, target: tuple = None) -> bool:
        if target == None:
            target = self.location
        if self.grid.is_location_in_grid(target):
            return True
        return False

    def is_blocking(self, value : bool = None):
        """
        Checks if actor blocks a field for other actors
        :type value: true: is blocked,  false: is not blocked
        :return : true if actor blocks a field
        """
        if value is not None:
            self._blocked = value
        return self._blocked

    def set_unblocked(self):
        """
        Unsets the blocked-status of actor. If blocked is true, the field can't be passed by another actor
        """
        self._blocked = False

    def __flip_x__(self):
        """
        Doesn't change status.
        Used for flipping all sprites according to flipped state
        """
        try:
            if not self._is_flipped:
                self.__grid__.repaint_area(pygame.Rect(self.image_rect))
                self._logging.debug(
                    "actor.flip_x() : Flipping " + self.title + " image number " + str(self._image_index))
                self.image = pygame.transform.flip(self._original_images[self._image_index], False, False)
                self.__grid__.repaint_area(pygame.Rect(self.image_rect))
            else:
                self.__grid__.repaint_area(pygame.Rect(self.image_rect))
                self.image = pygame.transform.flip(self._original_images[self._image_index], True, False)
                self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        except IndexError:
            self._logging.warning("actor.flip_x : Index Error in __flip_x__() in: Image Index is out of bounds")

    def flip_x(self):
        """
        Mirrors image over y-axis. x-Coordinates are flipped.
        Actor is turned by 180 degrees.
        is_rotatable should be False.
        """
        if not self._is_flipped:
            self._is_flipped = True
        else:
            self._is_flipped = False
        self.__flip_x__()
        self.turn_left(180)

    @property
    def image(self):
        """
        Gets the actual image of the actor.
        """
        return self._image

    @image.setter
    def image(self, value: str):
        """
        sets the actual image
        :param value: the path to the image
        """
        self._image = value

    def image_next(self):
        """
        switches to the next image by image_index
        """
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        if self._image_index < self._original_images.__len__() - 1:
            self._image_index = self._image_index + 1
        else:
            self._image_index = 0
        self.image = self._original_images[self._image_index]
        self.draw()
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        self._logging.debug("actor.image_next() : Image Index:" + str(self._image_index) + "/" + str(
            self._original_images.__len__() - 1))

    @property
    def is_rotatable(self):
        """
        : return true if actor-image is rotatable, else false
         """
        return self._is_rotatable

    @is_rotatable.setter
    def is_rotatable(self, value: bool):
        """
        sets the actual image
        :param value: the path to the image
        """
        self.__rotatable = value

    def set_rotatable(self):
        """
        Sets the actor-image rotatable.
        """
        self._is_rotatable = True

    def set_text(self, text, size):
        myfont = pygame.font.SysFont("monospace", size)
        label = myfont.render(text, 1, (0, 0, 0))
        self.image.blit(label, (0, 0))
        self._original_images[0] = self._image
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))

    def get_all_actors_at_location(self, class_name, location=None):
        if location == None:
            location = self.location
        actors = self.grid.get_actors_at_location(location, class_name)
        if self in actors:
            actors.remove(self)
        return actors

    def get_actor_at_location(self, class_name, location=None) -> list:
        """
        Get all actors at a specific location
        """
        if location == None:
            location = self.location
        actors_at_location = self.get_all_actors_at_location(class_name, location)
        if actors_at_location:
            return actors_at_location[0]

    def get_actor_in_front(self, class_name, location=None) -> list:
        """
        Get all actors at a specific location
        """
        location=self.look_forward()
        actors_at_location = self.get_all_actors_at_location(class_name, location)
        if actors_at_location:
            return actors_at_location[0]

    def __image_transform__(self, index: int, img_action: str, size: str = None):
        """
        Should be called before main-loop
        :type size: img_acton : "scale" -> data : location
        """
        img_action=str.lower(img_action)
        cell_size = self.grid.cell_size
        if img_action == "scale":
            if size is None:
                size = (cell_size, cell_size)
            self._original_images[index] = pygame.transform.scale(self._original_images[index],
                                                                      (size[0], size[1]))
        elif img_action == "center":
            cropped_surface = pygame.Surface((cell_size, cell_size), pygame.SRCALPHA, 32)
            width = self._original_images[self._image_index].get_width()
            height = self._original_images[self._image_index].get_height()
            x_pos = (self.grid.cell_size - width) / 2
            y_pos = (self.grid.cell_size - height) / 2
            cropped_surface.blit(self._original_images[self._image_index], (x_pos, y_pos),
                                 (0, 0, cell_size, cell_size))
            self._original_images[self._image_index] = cropped_surface
        elif img_action == "do_nothing":
            self._original_images[self._image_index] = self._original_images[self._image_index]

    @property
    def image_rect(self):
        """
            :return: The surrounding Rectangle used for redrawing and image manipulation
        """
        try:
            left, top = self.__get_image_coordinates_in_pixels__()
            width = self._image.get_width()
            height = self._image.get_height()
        except AttributeError:
            left, top, width, height = 0, 0, 0, 0
        return pygame.Rect(left, top, width, height)

    def __image_rect__(self, location=None):
        """
            the image rect when drawn to another location
        """
        if location == None:
            location = self.location
        try:
            width = self._image.get_width()
            height = self._image.get_height()
            left, top = self.__get_image_coordinates_in_pixels__(location)
        except AttributeError:
            left, top, width, height = 0, 0, 0, 0
        return pygame.Rect(left, top, width, height)

    def __cell_rect__(self, location: tuple = None):
        if location == None:
            location = self.location
        return self.grid.cell_rect(location)

    def __custom_rect__(self, location: tuple = None):
        if location is None:
            location = self.location
        width = self.__bounding_box_size__[0]
        height = self.__bounding_box_size__[1]
        left, top = (location[0] - self.__bounding_box_size__[0] / 2, location[1] - self.__bounding_box_size__[1] / 2)
        return pygame.Rect(left, top, width, height)

    def bounding_box(self, location: tuple = None):
        """
        :return: The surrounding Rectangle if actor is placed at location
        """
        if location == None:
            location = self.location
        if self.__bounding_box_type__ == "image":
            rect = self.__image_rect__(location)
            return rect
        if self.__bounding_box_type__ == "custom":
            rect = self.__custom_rect__(location)
            return rect
        else:
            return self.__cell_rect__(location)

    def set_bounding_box_size(self, value):
        self.__bounding_box_type__ = "custom"
        self.__bounding_box_size__ = value

    def __get_image_coordinates_in_pixels__(self, location: tuple = None):
        """
        Gets coordinates of top-left image position
        :return: (x-coordinate, y-coordinate)
        """
        if location == None:
            column = self.location[0]
            row = self.location[1]
        else:
            column = location[0]
            row = location[1]
        cell_margin = self.__grid__.cell_margin
        cell_size = self.__grid__.cell_size
        if self._image.get_width() > cell_size:
            overlapping_x = (self._image.get_width() - cell_size) / 2
        else:
            overlapping_x = 0

        if self._image.get_height() > cell_size:
            overlapping_y = (self._image.get_height() - cell_size) / 2
        else:
            overlapping_y = 0

        cell_left = cell_margin + (cell_margin + cell_size) * column - overlapping_x
        cell_top = cell_margin + (cell_margin + cell_size) * row - overlapping_y
        return cell_left, cell_top

    def get_neighbour_cells(self) -> list:
        """
        :rtype: all neighbour cells in a list.
        """
        locations = []
        y_pos = self.location[0]
        x_pos = self.location[1]
        locations.append([x_pos + 1, y_pos])
        locations.append([x_pos + 1, y_pos + 1])
        locations.append([x_pos, y_pos + 1])
        locations.append([x_pos - 1, y_pos + 1])
        locations.append([x_pos - 1, y_pos])
        locations.append([x_pos - 1, y_pos - 1])
        locations.append([x_pos, y_pos - 1])
        locations.append([x_pos + 1, y_pos - 1])
        return locations

    def listen(self, key, data):
        """ Should be overwritten in child-classes
        """
        pass

    def get_location(self) -> list:
        """
        Returns the location of actor
        :return: the location as tuple
        """
        return self.location

    @property
    def location(self) -> list:
        """
        returns the location of object
        """
        return self._location

    @location.setter
    def location(self, value: list):
        """
        Sets the location
        :type value: tuple with x and y-coordinate
        """
        self._logging.debug("actor.location: Location set")
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        self._location = value
        self.__grid__.repaint_area(pygame.Rect(self.image_rect))
        self.__grid__.__update__(no_logic=True)

    def set_x(self, x):
        """
        :param x: the x-coordinate
        """
        self.location[0] = x

    def set_y(self, y):
        """
        :param y: the y-coordinate
        """
        self.location[1] = y

    def get_x(self):
        """
        :param x: the x-coordinate
        """
        return self.location[0]

    def get_y(self):
        """
        :param y: the y-coordinate
        """
        return self.location[1]

    def setup(self):
        """
        Should be overwritten by child-classes
        """
        pass

    def __deprecated_set_location__(self, location):
        """
        sets the location
        deprecated: use location-proberty instead
        :param location:
        :return:
        """
        self.location = location

    def set_direction(self, degrees: int):
        """
        Sets the direction of actor
        """
        if degrees == "right":
            degrees = 0
        if degrees == "left":
            degrees = 180
        if degrees == "up":
            degrees = 90
        if degrees == "down":
            degrees = 270
        self.direction = degrees
        self._logging.debug("Richtung:" + str(self.direction))

    def turn_left(self, degrees: int = 90):
        """
        Turns the actor left by x degress
        """
        direction = self.direction + degrees
        self.direction = direction
        self._logging.debug("Richtung:" + str(self.direction))

    def turn_right(self, degrees: int = 90):
        """
        Turns the actor right by x degress
        """
        direction = self.direction - degrees
        self.direction = direction
        self._logging.debug("Richtung:" + str(self.direction))

    def move(self, distance: int = 1, move: bool = True):
        """
        Moves actor by x steps
        :param distance : number of steps the actor should step forward.
        """
        target = self.look_forward(distance)
        self._logging.info(
            "actor" + str(self) + ".move(): ...try to move from" + str(self.location) + " to target" + str(
                target) + " direction:" + str(self.direction))
        return self.move_to(target, move)

    def move_to(self, target: tuple, move: bool = True):
        if self.is_valid_target(target):
            if move is True:
                self.location = target
            return True
        else:
            return False

    def move_back(self, distance: int = 1, move: bool = True):
        """
        Moves actor by x steps
        :param distance : number of steps the actor should step forward.
        """
        target = self.look_back(distance)
        return self.move_to(target, move)

    def move_right(self, distance=1, move: bool = True):
        """
        Moves the actor one step right
        Sets the direction to 0° (East)
        """
        self.direction = 0
        return self.move(distance, move)

    def move_left(self, distance=1, move: bool = True):
        """
        Moves the actor one step left
        Sets the direction to 180° (West)
        """
        self.direction = 180
        self.move(distance, move)

    def move_down(self, distance=1, move: bool = True):
        """
        Moves the actor one step down
        Sets the direction to 170° (South)
        :type move: object
        """
        self.direction = 270
        return self.move(distance, move)

    def move_up(self, distance=1, move: bool = True):
        """
        Moves the actor one step down
        Sets the direction to 170° (South)
        """
        self.direction = 90
        return self.move(distance, move)

    def look_forward(self, distance: int = 1) -> tuple:
        """
        looks x steps forward
        :param distance : Number of steps to look forward
        :return location : Location as tuple (x_cell, y_cell)
        """
        self._logging.debug("actor.look_forward() : Location:" + str(self.location) + "Direction" + str(self.direction))
        loc_x = round(self.location[0] + math.cos(math.radians(self.direction)) * distance)
        loc_y = round(self.location[1] - math.sin(math.radians(self.direction)) * distance)
        return loc_x, loc_y

    def look_back(self, distance: int = 1) -> tuple:
        """
        looks x steps back
        :param distance : Number of steps to look forward
        :return location : Location as tuple (x_cell, y_cell)
        """
        self._logging.debug("actor.look_forward() : Location:" + str(self.location) + "Direction" + str(self.direction))
        loc_x = round(self.location[0] - math.cos(math.radians(self.direction)) * distance)
        loc_y = round(self.location[1] + math.sin(math.radians(self.direction)) * distance)
        return loc_x, loc_y

    def look_up(self, distance: int = 1) -> tuple:
        """
        looks x steps up
        :param distance : Number of steps to look forward
        :return location : Location as tuple (x_cell, y_cell)
        """
        self._logging.debug("actor.look_forward() : Location:" + str(self.location) + "Direction" + str(self.direction))
        loc_x = self.location[0]
        loc_y = self.location[1] - distance
        return loc_x, loc_y

    def look_down(self, distance: int = 1) -> tuple:
        """
        looks x steps down
        :param distance : Number of steps to look forward
        :return location : Location as tuple (x_cell, y_cell)
        """
        self._logging.debug("actor.look_forward() : Location:" + str(self.location) + "Direction" + str(self.direction))
        loc_x = self.location[0]
        loc_y = self.location[1] + distance
        return loc_x, loc_y

    def look_left(self, distance: int = 1) -> tuple:
        """
        looks x steps left
        :param distance : Number of steps to look forward
        :return location : Location as tuple (x_cell, y_cell)
        """
        self._logging.debug("actor.look_forward() : Location:" + str(self.location) + "Direction" + str(self.direction))
        loc_x = self.location[0] - distance
        loc_y = self.location[1]
        return loc_x, loc_y

    def look_right(self, distance: int = 1) -> tuple:
        """
        looks x steps right
        :param distance : Number of steps to look forward
        :return location : Location as tuple (x_cell, y_cell)
        """
        self._logging.debug("actor.look_forward() : Location:" + str(self.location) + "Direction" + str(self.direction))
        loc_x = self.location[0] + distance
        loc_y = self.location[1]
        return loc_x, loc_y

    def is_valid_move(self, move_func=None, distance=1):
        if move_func == None:
            move_func = self.move
        return move_func(distance, False)

    def is_valid_target(self, target: tuple = None):
        """
        Checks is a move to a specific target is valid
        :return true if move is legal, else false
        """
        valid = False
        if target is None:
            target = self.look_forward()
        if self.is_in_grid(target):
            self._logging.debug("actor.is_valid_target() : target:" + str(target) + ",true")
            valid = True
        else:
            self._logging.debug("actor.is_valid_target() : target:" + str(target) + ",false")
            valid = False
        # check if target is not blocked
        actors_at_position = self.__grid__.get_actors_at_location(target)
        for actor in actors_at_position:
            if actor.is_blocking():
                valid = False
        return valid

    @property
    def has_image(self):
        """
        Checks if actor has an image
        :return: true if actor has an image, else false
        """
        return self._has_image

    @has_image.setter
    def has_image(self, value):
        self._has_image = value

    def draw(self):
        """
        Draws the actor to the grid
        """
        if self.has_image:
            cell_left, cell_top = self.__get_image_coordinates_in_pixels__()
            self.__flip_x__()
            self.__rotate__(self.direction)
            if self.grid._show_bounding_boxes:
                pygame.draw.rect(self._image, (255, 0, 0),
                                 (0, 0, self.bounding_box().width, self.bounding_box().height), 2)
            if self.grid._show_direction_marker:
                # self._logging.debug("actor.draw() - Draw line from"+str(self._image.get_rect().center)+" to "+str(self._image.get_rect().topleft))
                x = round(
                    self.bounding_box().width / 2 + math.cos(math.radians(self.direction)) * self.bounding_box().width)
                y = round(self.bounding_box().height / 2 - math.sin(
                    math.radians(self.direction)) * self.bounding_box().height)
                pygame.draw.line(self.image, (255, 0, 0), (self.bounding_box().width / 2,
                                                           self.bounding_box().height / 2), (x, y), 2)
            self.grid.grid_surface.blit(self._image, (cell_left, cell_top))

    def remove(self):
        """
        removes the actor from grid
        """
        self._remove_from_grid()
        del (self)

    def _remove_from_grid(self):
        """
        removes the actor from grid
        """
        self.grid._actors.remove(self)
        self.__grid__ = None

    @property
    def grid(self):
        return self.__grid__



from codecs import open
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))


# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
	long_description = f.read()

setup(
    name='gamegridp',
    version='0.4.0.0',
    description='A gamegrid clone for python',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=['game', 'education'],  # arbitrary keywords
    author='Andreas Siebel',
    author_email='andreas.siebel@it-teaching.de',
    url='https://github.com/asbl/gamegridp',  # use the URL to the github repo
    download_url='https://github.com/asbl/gamegridp/archive/master.zip',  # I'll explain this in a second
    license="MIT",
    classifiers=[
        "License :: MIT",
        "Programming Language :: Python",
        "Development Status :: 3 - ALPHA",
        "Intended Audience :: Developers",
        "Topic :: Education",
    ],
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required
    package_data={'gamegridp': ['data/*.png']},
    install_requires=['pygame'],
)


import os
import pygame


class Toolbar(object):
    def __init__(self, grid):
        self.width = 200
        self.posx = 0
        self.grid = grid
        self.height = 0
        self.elements = []


    def set_height(self, height):
        self.height=height

    def set_posx(self, posx):
        self.posx=posx

    def draw(self):
        """
        Creates a toolbar on the left side of the window
        """
        toolbar = pygame.Surface((self.width, self.height))
        toolbar.fill((255, 255, 255))
        i = 0
        height = 0
        for element in self.elements:
            pygame.screen.blit(element.get_surface(), (self.posx, height))
            height = height + element.height


    def add_button(self, text, img_path=None):
        """
        adds a button to toolbar
        :param img_path: image button
        :param text: button text. This is also the text for the data variable in listen(event,data)
        :return:
        """
        button = ToolbarButton(self.width,20, img_path=img_path, text=text)
        self.elements.append(button)
        return button

    def __elements_height__(self):
        height=0
        for element in self.elements:
            height += element.height
        return height

    def listen(self, event, position : tuple):
        if event == "mouse_left":
            height=0
            if not position[1] > self.__elements_height__():
                for element in self.elements:
                    if height+element.height>position[1]:
                        return element.listen(event, position)
                    else :
                        height = height + element.height
        else:
            return "no toolbar event"

class ToolbarElement():
    def __init__(self):
        self.height = 20
        self.surface = None
        self.title=""
        self.event =  "no event"

    def get_surface(self):
        return self.surface

    def listen(self, event, position : tuple):
        return self.event


class ToolbarButton(ToolbarElement):

    def __init__(self, width, height, text, img_path):
        super().__init__()
        package_directory = os.path.dirname(os.path.abspath(__file__))
        myfont = pygame.font.SysFont("monospace", 15)
        button = pygame.Surface((width, 20))
        button.fill((255, 255, 255))
        label = myfont.render(text, 1, (0, 0, 0))

        if img_path != None:
            image = pygame.image.load(img_path)
            image = pygame.transform.scale(image, (20, 20))
            button.blit(image, (5, 5))
            button.blit(label, (25, 0))
        else:
            button.blit(label, (0, 0))

        self.surface=button
        self.event=text

    def listen(self, event, position : tuple):
        return self.event





import os
import pygame

class Console(object):
    def __init__(self, grid, lines):
        self.lines = 5
        self.grid = grid
        self.height = self.lines * 20
        self.text_queue=[]
        self.posy = 0

    def set_width(self, width):
        self.width= width

    def set_posy(self, posy):
        self.posy = posy

    def draw(self):
        console = pygame.Surface((self.width, self.lines*20))
        package_directory = os.path.dirname(os.path.abspath(__file__))
        myfont = pygame.font.SysFont("monospace", 15)
        console.fill((255, 200, 255))

        for i, text in enumerate(self.text_queue):
            line = pygame.Surface((self.width, 20))
            line.fill((255,255,200))
            label = myfont.render(text, 1, (0, 0, 0))
            line.blit(label, (0, 0))
            pygame.screen.blit(line, (0, self.posy+i*20))

    def print(self, text):
        self.text_queue.append(text)
        print (self.text_queue)
        if len(self.text_queue) > self.lines:
            self.text_queue.pop(0)


